@@ -1,6 +1,6 @@
 """
 Async DRACOON API wrapper based on httpx and pydantic
-V1.0.0
+V1.2.0
 (c) Octavio Simone, November 2021 
 
 Collection of DRACOON API calls
@@ -15,6 +15,10 @@ All requests with bodies use generic params variable to pass JSON body
 
 """
 
+
+import logging
+import asyncio
+
 from .crypto.models import PlainUserKeyPairContainer
 from .downloads import DRACOONDownloads
 from .public import DRACOONPublic
@@ -30,21 +34,32 @@ from .settings import DRACOONSettings
 from .reports import DRACOONReports
 from .crypto import decrypt_private_key
 from .logger import create_logger
-import asyncio
-import logging
+from .errors import CryptoMissingFileKeyrError, CryptoMissingKeypairError, HTTPNotFoundError, InvalidFileError, InvalidPathError, ClientDisconnectedError
+
 
 
 class DRACOON:
     """ DRACOON main API wrapper with all adapters to specific endpoints """ 
 
-    def __init__(self, base_url: str, client_id: str = 'dracoon_legacy_scripting', client_secret: str = '', log_file: str = 'dracoon.log', log_level = logging.INFO, log_stream: bool = False):
+    def __init__(self, base_url: str, client_id: str = 'dracoon_legacy_scripting', client_secret: str = '', log_file: str = 'dracoon.log', 
+                 log_level = logging.INFO, log_stream: bool = False, raise_on_err: bool = False):
         """ intialize with instance information: base DRACOON url and OAuth app client credentials """
-        self.client = DRACOONClient(base_url=base_url, client_id=client_id, client_secret=client_secret)
+        self.client = DRACOONClient(base_url=base_url, client_id=client_id, client_secret=client_secret, raise_on_err=raise_on_err)
 
         self.logger = create_logger(log_file=log_file, log_level=log_level, log_stream=log_stream)
         self.logger.info("Created DRACOON client.")
         self.plain_keypair = None
         self.user_info =  None
+        self.nodes = None
+        self.users = None
+        self.user = None
+        self.groups = None
+        self.downloads = None
+        self.settings = None
+        self.reports = None
+        self.eventlog = None
+        self.shares = None
+        self.uploads = None
 
 
     async def connect(self, connection_type: OAuth2ConnectionType = OAuth2ConnectionType.auth_code, username: str = None, password: str = None, auth_code = None):
@@ -116,7 +131,7 @@ class DRACOON:
         """ get user keypair """
         if not self.client.connection:
             self.logger.error("DRACOON client not connected: Keypair not retrieved.")
-            raise ValueError('DRACOON client not connected.')
+            raise ClientDisconnectedError()
 
         enc_keypair = await self.user.get_user_keypair()
       
@@ -129,18 +144,24 @@ class DRACOON:
 
         return plain_keypair
 
-    async def upload(self, file_path: str, target_path: str, display_progress: bool = False):
+    async def upload(self, file_path: str, target_path: str, resolution_strategy: str = 'autorename', display_progress: bool = False, raise_on_err: bool = False):
         """ upload a file to a target """
         if not self.client.connection:
             self.logger.error("DRACOON client not connected: Upload failed.")
-            raise ValueError('DRACOON client not connected.')
+            err = ClientDisconnectedError(message="DRACOON client not connected.")
+            await self.client.handle_generic_error(err=err)
+            
+        if self.client.raise_on_err:
+            raise_on_err = True
 
-        node_info = await self.nodes.get_node_from_path(target_path)
+        node_info = await self.nodes.get_node_from_path(path=target_path, raise_on_err=raise_on_err)
         
         if not node_info:
             self.logger.critical('Upload failed: Invalid target path.')
-            self.logger.debug('Node %s not found.', target_path)
-            raise ValueError('Node %s not found.', target_path)
+            msg = 'Node %s not found.', target_path
+            self.logger.debug(msg)
+            err = InvalidPathError(message=msg)
+            await self.client.handle_generic_error(err=err)
         
         file_name = file_path.split('/')[-1]
         
@@ -163,7 +184,7 @@ class DRACOON:
         self.logger.debug("Using S3 storage: %s", use_s3_storage)
             
         upload_channel_payload = self.nodes.make_upload_channel(parent_id=target_id, name=file_name, direct_s3_upload=use_s3_storage)
-        upload_channel = await self.nodes.create_upload_channel(upload_channel_payload)
+        upload_channel = await self.nodes.create_upload_channel(upload_channel=upload_channel_payload, raise_on_err=raise_on_err)
     
 
         self.logger.debug("Created upload channel: %s", upload_channel.uploadId)
@@ -171,37 +192,43 @@ class DRACOON:
 
         # crypto upload 
         if is_encrypted and self.check_keypair() and not use_s3_storage:       
-            upload = await self.uploads.upload_encrypted(file_path=file_path, user_id=user_id, upload_channel=upload_channel, plain_keypair=self.plain_keypair)
+            upload = await self.uploads.upload_encrypted(file_path=file_path, user_id=user_id, upload_channel=upload_channel, 
+                                                         plain_keypair=self.plain_keypair, resolution_strategy=resolution_strategy, raise_on_err=raise_on_err)
         elif is_encrypted and self.check_keypair() and use_s3_storage:
-            upload = await self.nodes.upload_s3_encrypted(file_path=file_path, upload_channel=upload_channel, plain_keypair=self.plain_keypair, display_progress=display_progress)
+            upload = await self.nodes.upload_s3_encrypted(file_path=file_path, upload_channel=upload_channel, plain_keypair=self.plain_keypair, 
+                                                          display_progress=display_progress, resolution_strategy=resolution_strategy, raise_on_err=raise_on_err)
         elif is_encrypted and not self.check_keypair():
             self.logger.critical("Upload failed: Keypair not unlocked.")
-            raise ValueError('DRACOON crypto upload requires unlocked keypair. Please unlock keypair first.')
+            raise CryptoMissingKeypairError('DRACOON crypto upload requires unlocked keypair. Please unlock keypair first.')
         # unencrypted upload
         elif not is_encrypted and not use_s3_storage:
-            upload = await self.uploads.upload_unencrypted(file_path=file_path, upload_channel=upload_channel)
+            upload = await self.uploads.upload_unencrypted(file_path=file_path, upload_channel=upload_channel, resolution_strategy=resolution_strategy, raise_on_err=raise_on_err)
         elif not is_encrypted and use_s3_storage:
-            upload = await self.nodes.upload_s3_unencrypted(file_path=file_path, upload_channel=upload_channel, display_progress=display_progress)
+            upload = await self.nodes.upload_s3_unencrypted(file_path=file_path, upload_channel=upload_channel, 
+                                                            display_progress=display_progress, resolution_strategy=resolution_strategy,raise_on_err=raise_on_err)
 
         self.logger.info("Upload completed.")
         
         if upload: return upload
 
-    async def download(self, file_path: str, target_path: str, display_progress: bool = False):
+    async def download(self, file_path: str, target_path: str, display_progress: bool = False, raise_on_err: bool = False):
         """ download a file to a target """
 
         if not self.client.connection:
             await self.client.http.aclose()
             self.logger.error("DRACOON client not connected: Download failed.")
-            raise ValueError('DRACOON client not connected.')
+            raise ClientDisconnectedError(message='DRACOON client not connected.')
 
-        node_info = await self.nodes.get_node_from_path(file_path)
+        if self.client.raise_on_err:
+            raise_on_err = True
+
+        node_info = await self.nodes.get_node_from_path(path=file_path, raise_on_err=raise_on_err)
         
         if not node_info:
             self.logger.error("Download failed: file does not exist.")
             self.logger.debug("File %s not found", file_path)
-            await self.client.http.aclose()
-            raise ValueError('File does not exist.')
+            err = InvalidFileError(message='File does not exist.')
+            await self.client.handle_generic_error(err=err)
         
         node_id = node_info.id
 
@@ -212,22 +239,36 @@ class DRACOON:
         self.logger.debug("Destination: %s", target_path)
         self.logger.debug("Encrypted: %s", is_encrypted)
 
-        dl_token_res = await self.nodes.get_download_url(node_id=node_id)
+        dl_token_res = await self.nodes.get_download_url(node_id=node_id, raise_on_err=raise_on_err)
 
         download_url = dl_token_res.downloadUrl
 
         self.logger.debug("Download URL (redacted): %s", download_url[:-4])
 
         if not is_encrypted:
-            await self.downloads.download_unencrypted(download_url=download_url, target_path=target_path, node_info=node_info, display_progress=display_progress)
+            await self.downloads.download_unencrypted(download_url=download_url, target_path=target_path, node_info=node_info, 
+                                                      display_progress=display_progress, raise_on_err=raise_on_err)
         elif is_encrypted and self.check_keypair():
-            file_key = await self.nodes.get_user_file_key(node_id)
-            await self.downloads.download_encrypted(download_url=download_url, target_path=target_path, node_info=node_info, plain_keypair=self.plain_keypair, file_key=file_key, display_progress=display_progress)
+            try:
+                file_key = await self.nodes.get_user_file_key(node_id, raise_on_err=True)
+                await self.downloads.download_encrypted(download_url=download_url, target_path=target_path, node_info=node_info, 
+                                                    plain_keypair=self.plain_keypair, file_key=file_key, display_progress=display_progress, raise_on_err=raise_on_err)
+            except HTTPNotFoundError:
+                raise CryptoMissingFileKeyrError(message=f'No file key for node {node_id}')
+                
         elif is_encrypted and not self.check_keypair():
-            raise ValueError('Keypair must be entered for encrypted nodes.')
+            raise CryptoMissingKeypairError(message='Keypair must be entered for encrypted nodes.')
 
 
     def get_code_url(self) -> str:
         """ get code url for authorization code flow """
         self.logger.info("Getting authorization URL.")
-        return self.client.get_code_url()
\ No newline at end of file
+        return self.client.get_code_url()
+
+    def batch_process(self, coro_list, batch_size: int = 5):
+        """ 
+        helper method which returns a generator for a list 
+        of couroutines 
+        utility to process multiple requests async
+        """ 
+        return (coro_list[i:i + batch_size]  for i in range(0, len(coro_list), batch_size))
