@@ -1,6 +1,6 @@
 """
 Async DRACOON nodes adapter based on httpx and pydantic
-V1.0.0
+V1.2.0
 (c) Octavio Simone, November 2021 
 
 Collection of DRACOON API calls for nodes management
@@ -19,8 +19,10 @@ import os
 import math
 from pathlib import Path
 from typing import List, Union
-import httpx
 import logging
+import asyncio
+
+import httpx
 from pydantic import validate_arguments
 from tqdm import tqdm
 
@@ -28,6 +30,8 @@ from dracoon.crypto import FileEncryptionCipher, encrypt_bytes, encrypt_file_key
 from dracoon.crypto.models import FileKey, PlainUserKeyPairContainer
 from dracoon.groups.models import Expiration
 from dracoon.client import DRACOONClient, OAuth2ConnectionType
+from dracoon.errors import (InvalidClientError, ClientDisconnectedError, InvalidPathError, 
+                           FileConflictError, InvalidFileError, InvalidArgumentError)
 from .models import (CompleteS3Upload, ConfigRoom, CreateFolder, CreateRoom, CreateUploadChannel, 
                      GetS3Urls, LogEventList, MissingKeysResponse, Node, Permissions, ProcessRoomPendingUsers, S3Part, 
                      SetFileKeys, SetFileKeysItem, TransferNode, CommentNode, RestoreNode, UpdateFile, UpdateFiles, 
@@ -41,6 +45,7 @@ from .responses import (Comment, CommentList, CreateFileUploadResponse, DeletedN
 # constants for uploads 
 CHUNK_SIZE = 33554432
 MAX_CHUNKS = 9999
+POLL_WAIT = 0.1
 
 class DRACOONNodes:
 
@@ -52,7 +57,8 @@ class DRACOONNodes:
     def __init__(self, dracoon_client: DRACOONClient):
         """ requires a DRACOONClient to perform any request """
         if not isinstance(dracoon_client, DRACOONClient):
-            raise TypeError('Invalid DRACOON client format.')
+            raise InvalidClientError(message='Invalid client')
+
         if dracoon_client.connection:
             self.dracoon = dracoon_client
             self.api_url = self.dracoon.base_url + self.dracoon.api_base_url + '/nodes'
@@ -64,9 +70,7 @@ class DRACOONNodes:
             self.logger.debug("DRACOON nodes adapter created.")
         else:
             self.logger.error("DRACOON client error: no connection. ")
-            err = ValueError(
-                'DRACOON client must be connected: client.connect()')
-            self.dracoon.handle_generic_error(err)
+            raise ClientDisconnectedError(message='DRACOON client must be connected: client.connect()')
     
     async def upload_bytes(self, file_obj, progress: tqdm):
         """ async iterator to stream byte upload """
@@ -289,8 +293,8 @@ class DRACOONNodes:
         file = Path(file_path)
         
         if not file.is_file():
-            err = ValueError(f'A file needs to be provided. {file_path} is not a file.')
-            self.dracoon.handle_generic_error(err)
+            err = InvalidFileError(message=f'A file needs to be provided. {file_path} is not a file.')
+            await self.dracoon.handle_generic_error(err)
         
         filesize = os.stat(file_path).st_size
         file_name = file_path.split('/')[-1]
@@ -300,6 +304,10 @@ class DRACOONNodes:
 
         part_count = math.ceil(filesize / chunksize)
         
+        # handle 0KB files
+        if part_count == 0:
+            part_count = 1
+        
         if part_count == 1:
             chunksize = filesize
             s3_upload = self.make_get_s3_urls(first_part=1, last_part=part_count, chunk_size=chunksize)
@@ -319,8 +327,8 @@ class DRACOONNodes:
         self.logger.debug("Parts: %s", part_count)
 
         if len(s3_urls.urls) > MAX_CHUNKS:
-            err = ValueError(f'Maximum count of chunks ({MAX_CHUNKS}) exceeded.')
-            self.dracoon.handle_generic_error(err)
+            err = InvalidArgumentError(message=f'Maximum count of chunks ({MAX_CHUNKS}) exceeded.')
+            await self.dracoon.handle_generic_error(err)
             
        
         parts = []
@@ -402,12 +410,18 @@ class DRACOONNodes:
  
         await self.complete_s3_upload(upload_id=upload_channel.uploadId, upload=s3_complete, raise_on_err=raise_on_err)
         
+        time  = POLL_WAIT
+        
         while True:
             upload_status = await self.check_s3_upload(upload_id=upload_channel.uploadId, raise_on_err=raise_on_err)
             if upload_status.status == S3Status.done.value:
                 break
             if upload_status.status == S3Status.error.value:
                 break
+            # wait until next request
+            await asyncio.sleep(time)
+            # increase wait 
+            time *= 2
             
         return upload_status
         
@@ -420,13 +434,12 @@ class DRACOONNodes:
         if self.raise_on_err:
             raise_on_err = True
 
-        
         file = Path(file_path)
         
         # check if file is a file
         if not file.is_file():
-            err = ValueError(f'A file needs to be provided. {file_path} is not a file.')
-            self.dracoon.handle_generic_error(err)
+            err = InvalidFileError(message=f'A file needs to be provided. {file_path} is not a file.')
+            await self.dracoon.handle_generic_error(err)
         
         filesize = os.stat(file_path).st_size
         file_name = file_path.split('/')[-1]
@@ -439,7 +452,10 @@ class DRACOONNodes:
         # calculate required parts based on chunk size
         part_count = math.ceil(filesize / chunksize)
         
-
+        # handle 0KB file 
+        if part_count == 0:
+            part_count = 1
+        
         if part_count == 1:
             chunksize = filesize
             s3_upload = self.make_get_s3_urls(first_part=1, last_part=part_count, chunk_size=chunksize)
@@ -459,8 +475,8 @@ class DRACOONNodes:
         self.logger.debug("Parts: %s", part_count)
 
         if len(s3_urls.urls) > MAX_CHUNKS:
-            err = ValueError(f'Maximum count of chunks ({MAX_CHUNKS}) exceeded.')
-            self.dracoon.handle_generic_error(err)
+            err = InvalidArgumentError(message=f'Maximum count of chunks ({MAX_CHUNKS}) exceeded.')
+            await self.dracoon.handle_generic_error(err)
             
         parts = []
         
@@ -561,17 +577,21 @@ class DRACOONNodes:
  
         await self.complete_s3_upload(upload_id=upload_channel.uploadId, upload=s3_complete, raise_on_err=raise_on_err)
         
+        time  = POLL_WAIT 
+        
         while True:
             upload_status = await self.check_s3_upload(upload_id=upload_channel.uploadId, raise_on_err=raise_on_err)
             if upload_status.status == S3Status.done.value:
                 break
             if upload_status.status == S3Status.error.value:
                 break
+            # wait until next request
+            await asyncio.sleep(time)
+            # increase wait 
+            time *= 2
             
         return upload_status
             
-        
-        
     async def check_s3_upload(self, upload_id: str, raise_on_err: bool = False) -> S3FileUploadStatus:
         """ check status of S3 upload """
         if not await self.dracoon.test_connection() and self.dracoon.connection:
@@ -1407,7 +1427,7 @@ class DRACOONNodes:
         if classification: room["classification"] = classification
 
         if not admin_ids and not admin_group_ids and inherit_perms is False:
-            raise ValueError('Room admin required: Please provide at least one room admin.')
+            raise InvalidArgumentError(message='Room admin required: Please provide at least one room admin.')
 
         return CreateRoom(**room)
 
